@@ -7,7 +7,7 @@
 #' @importFrom lifecycle deprecate_soft
 #' @importFrom lifecycle deprecated
 #' @importFrom dplyr .data sym
-#' @importFrom ggplot2 after_stat
+#' @importFrom ggplot2 after_stat after_scale
 ## usethis namespace: end
 NULL
 
@@ -23,11 +23,6 @@ utils::globalVariables(c("prop"))
 # \lifecycle{defunct}
 # \lifecycle{archived}
 
-#' @importFrom magrittr %>%
-#' @export
-magrittr::`%>%`
-
-
 # from ggplot2 (but not exported by ggplot2)
 
 `%||%` <- function(a, b) {
